package com.fr.plugin.core;

import com.fr.data.NetworkHelper;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.mobile.terminal.TerminalHandler;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.login.TokenResource;
import com.fr.general.ComparatorUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.config.MyConfig;
import com.fr.stable.StringUtils;
import com.fr.stable.web.Device;
import com.fr.third.org.apache.commons.lang3.RandomStringUtils;
import com.fr.web.utils.WebUtils;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.zip.Deflater;

public class LoginFilter extends AbstractGlobalRequestFilterProvider {
    @Override
    public String filterName() {
        return "cloudPriFilter";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{
                "/decision/login",
                "/decision/url/mobile",
                "/decision/view/form",
                "/decision/view/report",
                "/decision/schedule/*"
        };
    }

    @Override
    public void init(FilterConfig filterConfig) {
        super.init(filterConfig);
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {
        try {
            if (ComparatorUtils.equalsIgnoreCase(req.getMethod(), "get")) {
                if (isLogin(req)) {
                    filterChain.doFilter(req, res);
                } else {
                    if (ComparatorUtils.equalsIgnoreCase(req.getParameter("isAdmin"), "1")) {
                        filterChain.doFilter(req, res);
                    } else {
                        getlogin(req, res);
                    }
                }
            } else {
                filterChain.doFilter(req, res);
            }
        } catch (Exception e) {
            e.printStackTrace();
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
    }

    private boolean isLogin(HttpServletRequest request) {
        String oldToken = TokenResource.COOKIE.getToken(request);
        return oldToken != null && checkTokenValid(request, (String) oldToken);
    }

    private boolean checkTokenValid(HttpServletRequest req, String token) {
        try {
            Device device = NetworkHelper.getDevice(req);
            LoginService.getInstance().loginStatusValid(token, TerminalHandler.getTerminal(req, device));
            return true;
        } catch (Exception ignore) {
        }
        return false;
    }

    //登录页面
    public static void getlogin(HttpServletRequest request, HttpServletResponse response) throws Exception {
        String callBackUrl = "";
        String callBack = request.getParameter("callBack");
        if (StringUtils.isNotBlank(callBack)) {
            callBackUrl = callBack;
        }
        String pathInfo = request.getPathInfo();
        String originalURL = WebUtils.getOriginalURL(request);
        if (pathInfo.contains("/schedule")) {
            callBackUrl = MyConfig.getInstance().getUrl() + pathInfo + "?" + request.getQueryString();
        }
        if (pathInfo.contains("/view/form")) {
            callBackUrl = MyConfig.getInstance().getUrl() + pathInfo + "?" + request.getQueryString();
        }
        if (pathInfo.contains("/view/report")) {
            callBackUrl = MyConfig.getInstance().getUrl() + pathInfo + "?" + request.getQueryString();
        }
        if (pathInfo.contains("url/mobile")) {
            callBackUrl = MyConfig.getInstance().getUrl() + "/url/mobile";
        }
        if (StringUtils.isNotBlank(callBackUrl)) {
            FineLoggerFactory.getLogger().error("--------获取地址：{} 回调地址:{} 已经添加到cookies", originalURL, callBackUrl);

            setCookie(response,"saml_callBack",callBackUrl);
        }
        MyConfig instance = MyConfig.getInstance();
        String servie = instance.getUrl();
        String requestURI = servie + "/url/saml_login_callback";
        String samlRequest = "";
        // 设置当前访问时间
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss'Z'");
        Date now = new Date();
        // 设置samlRequest信息
        String samlRequestString = new StringBuilder().append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><samlp:AuthnRequest xmlns:samlp=\"urn:oasis:names:tc:SAML:2.0:protocol\" ID=\"").append(RandomStringUtils.random(40, true, false)).append("\" Version=\"2.0\" IssueInstant=\"").append(fmt.format(now)).append("\" ProtocolBinding=\"urn:oasis:names.tc:SAML:2.0:bindings:HTTP-Redirect\" ProviderName=\"samldemo\" AssertionConsumerServiceURL=\"").append(requestURI).append("\"/>").toString();
        samlRequest = encoding(samlRequestString);
        String idm = instance.getAuthUrl();
        if (StringUtils.isBlank(idm)) {
            idm = "https://testlogin.idm.huafagroup.com:25443/siam/login";
        }
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("loginURL", idm);
        map.put("samlRequest", samlRequest);
        map.put("requestURI", requestURI);
        String errorTemplate = "/com/fr/plugin/web/LoginRequire.html";
        WebUtils.writeOutTemplate(errorTemplate, response, map);
    }
    private static void setCookie(HttpServletResponse response, String name, String value) {
        Cookie cookie = new Cookie(name, value);
        cookie.setPath("/");
        response.addCookie(cookie);
    }

    private static String encoding(String originalString) {

        String outputString = null;

        // 将原始包变成bin byte[]
        byte[] binByte = originalString.getBytes(StandardCharsets.UTF_8);

        // 将bin byte[]压缩
        byte[] output = new byte[1000];
        Deflater compresser = new Deflater(Deflater.BEST_COMPRESSION, true);
        compresser.setInput(binByte); // 要压缩的数据包
        compresser.finish(); // 完成
        compresser.deflate(output); // 压缩，返回的是数据包经过缩缩后的大小

        // 将压缩后的 bin byte[]变为 base64 byte[]
        byte[] base64Input = Base64.getEncoder().encode(output);
        // 将 base64 byte[]变为base64 String
        outputString = new String(base64Input, StandardCharsets.UTF_8);
        return outputString;

    }
}
