package com.fr.plugin.core;

import com.fr.base.FRContext;
import com.fr.base.ServerConfig;
import com.fr.decision.system.bean.message.MessageUrlType;
import com.fr.general.FRLogger;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.beans.MyOutputBean;
import com.fr.plugin.config.MyConfig;
import com.fr.plugin.dao.PushDao;
import com.fr.plugin.entitys.MyAgentEntity;
import com.fr.plugin.entitys.PushEntity;
import com.fr.plugin.utils.*;
import com.fr.schedule.base.constant.ScheduleConstants;
import com.fr.schedule.feature.output.OutputActionHandler;
import com.fr.stable.CodeUtils;
import com.fr.stable.StringUtils;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.third.org.apache.commons.lang3.RandomStringUtils;

import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.UUID;

public class MyOutputActionHandler extends OutputActionHandler<MyOutputBean> {

    private static final int NEWS_MESSAGE = 2;
    private static final int TERMINATE_TYPE = 107;
    private static final int LINKTYPE = 0;
    private static final int PUSH2PC = 1;

    @Override
    public void doAction(MyOutputBean myOutputBean, Map<String, Object> map) throws Exception {
        String[] files = (String[]) map.get(ScheduleConstants.OUTPUT_FILES);
        Object username = map.get("username");
        String[] names = new String[0];

        if (username != null) {
            String s = username.toString();
            if (StringUtils.isNotBlank(s)) {
                names = new String[]{
                        s
                };
            }
        }

        if (names.length == 0) {
            names = (String[]) map.get(ScheduleConstants.USERNAMES);
        }

        MyConfig instance = MyConfig.getInstance();
        /**
         *  {
         * text:"图文",
         value:"1"
         *  },
         *RepConfig{
         *RepConfig    text:"纯图",
         *RepConfig    value:"2"
         *RepConfig},
         *RepConfig{
         *RepConfig    text:"纯文字",
         *RepConfig    value:"3"
         *RepConfig}
         */
        MyAgentEntity agent = MyUtils.getAgentById(myOutputBean.getAgentId());
        if (agent == null) {
            FineLoggerFactory.getLogger().error("获取agentId为空{}", myOutputBean.getAgentId());
            return;
        }
        String secret = agent.getSecret();
        String sysKey = agent.getSysKey();
        int messageType = myOutputBean.getMessageType();
        int platform = myOutputBean.getPlatform();
        pushToDevice(myOutputBean, map, platform, messageType, userToStr(names), sysKey, secret);
    }

    private void pushToPc(MyOutputBean myOutputBean, Map<String, Object> map, String usersStr, String sysKey, String pushSecret, int messageType) throws Exception {
        String pcUrl = getResultUrl(map);
        //3.建立信息bean
        String content = myOutputBean.getContent();//消息内容-需要
        String title = myOutputBean.getSubject();//标题-需要

        if (messageType == 1) {
            PushObject bmm = new PushObject();
            bmm.setTitle(title);
            bmm.setSummary(content);
            bmm.setJumpUrl(pcUrl);//pc页面，
            String appUrl = getResultUrl(map);
            bmm.setExtras("{\"appUrl\":\"" + appUrl + "\"}");
            bmm.setCoverImgUrl("");
            bmm.setShareRange("0");
            String pushPc = myOutputBean.getPlatform() + "";//0pc&app 1pc 2 app
            if (com.fr.stable.StringUtils.isNotBlank(pushPc)) {
                bmm.setOpenType(pushPc);
            } else {
                bmm.setOpenType("3");
            }
//        String usersStr = this.userToStr(users);//转成user，
            bmm.setBatchToUsers(usersStr);
            String url = MyConfig.getInstance().getPushUrl();
            String pushImageAndTextUrl = url + "/mpm/v5/open/templatemsg/createAndPush";
            String accessToken = MyConfig.getInstance().getAccessToken();
            PushUtils.push(pushImageAndTextUrl, pushSecret, sysKey, accessToken, bmm, usersStr);
        } else if (messageType == 2) {
            String url = MyConfig.getInstance().getPushUrl();
//            PushUtils.pushImage(url, usersStr, title, "", )
        }
    }


    private String userToStr(String[] users) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < users.length; i++) {
            sb.append(users[i]);
            if (i != users.length - 1) {
                sb.append(",");
            }
        }
        return String.valueOf(sb);
    }

    private String convertPCAndMob(int plantform) {
        if (plantform == 0) {
            return "3";
        } else if (plantform == 1) {
            return "1";
        } else if (plantform == 2) {
            return "2";
        }
        return "1";
    }

    public void pushToDevice(MyOutputBean myOutputBean, Map<String, Object> map, int platform, int messageType, String usersStr, String sysKey, String pushSecret) throws Exception {
        String url = MyConfig.getInstance().getPushUrl();
        String pushImageAndTextUrl = url + "/mpm/v5/open/templatemsg/createAndPush";
        String accessToken = MyConfig.getInstance().getAccessToken();
        String pushImageUrl = url + "/mpm/v5/open/image/push";
        String pushTextUrl = url + "/mpm/v5/open/textmsg/push";
        String customizeLink = myOutputBean.getCustomizeLink();
        if (StringUtils.isNotBlank(customizeLink)) {
            if (!customizeLink.startsWith("http")) {
                customizeLink = "http://" + customizeLink;
            }
        }
        if (messageType == 1) {//图文
            String pcUrl = getResultUrl(map);
            if (StringUtils.isNotBlank(customizeLink)) {
                pcUrl = customizeLink;
            }
            String content = myOutputBean.getContent();//消息内容-需要
            String title = myOutputBean.getSubject();//标题-需要
            String picId = myOutputBean.getMediaId();//传入的图片id
            PushObject bmm = new PushObject();
            if (StringUtils.isNotBlank(picId)) {
                String picUrl = this.getPicUrl(picId);//获取图片的id
                bmm.setCoverImgUrl(picUrl);
            }
            bmm.setTitle(title);
            bmm.setSummary(content);
            bmm.setJumpUrl(pcUrl);//pc页面，
            String appUrl = createScheduleEntryUrl(myOutputBean, map);
            if (StringUtils.isNotBlank(customizeLink)) {
                appUrl = customizeLink;
            }
            bmm.setExtras("{\"appUrl\":\"" + appUrl + "\"}");
            bmm.setShareRange("0");
            //0pc&app 1pc 2 app
            bmm.setOpenType(convertPCAndMob(platform));
            bmm.setBatchToUsers(usersStr);
            PushUtils.push(pushImageAndTextUrl, pushSecret, sysKey, accessToken, bmm, usersStr);
        } else if (2 == messageType) {//单图
            PushImageBean pushImageBean = new PushImageBean();
            String picId = myOutputBean.getMediaId();//传入的图片id
            String picUrl = this.getPicUrl(picId);//获取图片的id
            pushImageBean.setImageurl(picUrl);
            pushImageBean.setFilterUser(usersStr);
            String media_id = PushUtils.pushTextOrImage(pushImageUrl, pushImageBean, sysKey, pushSecret);
//            FRContext.getLogger().info("推送成功id 为：" + media_id);
//            String pcUrl = getResultUrl(map);
//            if(StringUtils.isNotBlank(myOutputBean.getCustomizeLink())){
//                pcUrl=myOutputBean.getCustomizeLink();
//            }
//            String content = myOutputBean.getContent();//消息内容-需要
//            String title = myOutputBean.getSubject();//标题-需要
//            String picId = myOutputBean.getMediaId();//传入的图片id
//            String picUrl = this.getPicUrl(picId);//获取图片的id
//            PushObject bmm = new PushObject();
////            bmm.setTitle(title);
//
////            bmm.setJumpUrl(pcUrl);//pc页面，
////            String appUrl = createScheduleEntryUrl(myOutputBean, map);
////            if(StringUtils.isNotBlank(myOutputBean.getCustomizeLink())){
////                appUrl=myOutputBean.getCustomizeLink();
////            }
////            bmm.setExtras("{\"appUrl\":\"" + appUrl + "\"}");
//            bmm.setCoverImgUrl(picUrl);
//            bmm.setShareRange("0");
//            //0pc&app 1pc 2 app
//            bmm.setOpenType(convertPCAndMob(platform));
//            bmm.setBatchToUsers(usersStr);
//            PushUtils.push(pushImageAndTextUrl, pushSecret, sysKey, accessToken, bmm, usersStr);
        } else {//文字
            PushTextBean pushTextBean = new PushTextBean();
            String content = myOutputBean.getSubject() + " \n" + myOutputBean.getContent();//消息内容-需要
            pushTextBean.setText(content);
            pushTextBean.setFilterUser(usersStr);
            String media_id = PushUtils.pushTextOrImage(pushTextUrl, pushTextBean, sysKey, pushSecret);
////            FRContext.getLogger().info("推送成功id 为：" + media_id);
//            String pcUrl = getResultUrl(map);
//            String picId = myOutputBean.getMediaId();//传入的图片id
//            PushObject bmm = new PushObject();
////            if (StringUtils.isNotBlank(picId)) {
////                String picUrl = this.getPicUrl(picId);//获取图片的id
////                bmm.setCoverImgUrl(picUrl);
////            }
////            String picUrl = this.getPicUrl( picId);//获取图片的id
////            bmm.setTitle(title);
//            String content = myOutputBean.getContent();//消息内容-需要
//            String title = myOutputBean.getSubject();//标题-需要
//            bmm.setSummary(title+"\n"+content);
//            bmm.setTitle("");
////            bmm.setSummary(content);
////            if(StringUtils.isNotBlank(myOutputBean.getCustomizeLink())){
////                pcUrl=myOutputBean.getCustomizeLink();
////            }
//
////            bmm.setJumpUrl(pcUrl);//pc页面，
//            String appUrl = createScheduleEntryUrl(myOutputBean, map);
//            if (StringUtils.isNotBlank(customizeLink)) {
//                appUrl = customizeLink;
//            }
////            bmm.setExtras("{\"appUrl\":\"" + appUrl + "\"}");
//            bmm.setShareRange("0");
//            //0pc&app 1pc 2 app
//            bmm.setOpenType(convertPCAndMob(platform));
//            bmm.setBatchToUsers(usersStr);
//            PushUtils.push(pushImageAndTextUrl, pushSecret, sysKey, accessToken, bmm, usersStr);
        }
    }

    private String getPicUrl(String picId) {
        String reportServer = getUrl();
        StringBuffer sb = new StringBuffer();
        sb.append(reportServer).append("/url/getImage?imageId=").append(picId);
        return String.valueOf(sb);
    }

//    /**
//     * @param obj 前端定时任务信息
//     * @return 拼接报表打开的地址：已包含单点登录地址
//     * @throws Exception
//     */
//    private String getEntryUrl(Map<String,String> obj, boolean ispc) throws Exception {
//        String strTemp = "";
//        //1.获取当前报表服务器外网地址
//        String reportServer = getUrl();
//        //2.拼接报表地址路径
//        int linkType = obj.optInt("linkType");
//        String withLink = obj.optString("withLink");
//        String groupId = obj.optString("groupId");
//
//        if (linkType == LINKTYPE) {
//            String taskName = obj.optString("taskName");//任务名称
//            String fireTime = obj.optString("fireTime");
//            String currentRes = obj.optString("currentRes");
//            currentRes = CodeUtils.encodeURIComponent(currentRes);
//            boolean showType = (obj.optInt("showType") != 2);//显示类别
//            StringBuffer sb = new StringBuffer();
//            String op = ispc ? "sch" : "h5";
//            sb.append(reportServer)
//                    .append("?op=").append(op).append("&taskName=").append(taskName);
//            FRContext.getLogger().info("当前推送的是pc:" + ispc + " op=" + op + " taskName=" + taskName);
//            if (!ispc) {
//                sb.append("&cmd=entry_cprurl");
//            }
//            sb.append("&fireTime=").append(fireTime).append("&currentRes=").append(currentRes).append("&groupId=").append(groupId).append("&__bypagesize__=").append(showType);
//            strTemp = String.valueOf(sb);
//        } else {
//            URI reportServerTemp = new URI(reportServer);
//            withLink = reportServerTemp.resolve(withLink).toString();
//            if (withLink.contains("?")) {
//                withLink = withLink + "&groupId=" + groupId;
//            } else {
//                withLink = withLink + "?groupId=" + groupId;
//            }
//            strTemp = withLink;
//        }
//
//        //返回
//        strTemp = getAuthLoginUrl(strTemp);
//        FRContext.getLogger().info("当前推送完整url:" + strTemp);
//        return strTemp;
//    }


    public static String getUrl() {
        return MyConfig.getInstance().getUrl();
    }

    private JSONObject delImageSource(String title, String desc, String url, String appId, String pic) {
        JSONObject jsonObject = JSONObject.create();
        jsonObject.put("model", 2);//2：单条图文混排模板
        jsonObject.put("todo", 0);
        jsonObject.put("sourceid", "");
        JSONArray list = JSONArray.create();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        String time = dateFormat.format(new Date());
        JSONObject model = JSONObject.create();
        model.put("date", time);
        model.put("title", title);
        model.put("text", desc);
        model.put("zip", "");
        model.put("url", url);
        model.put("appid", appId);
        model.put("name", RandomStringUtils.randomAlphabetic(5) + ".png");
        model.put("pic", pic);
        list.add(model);
        jsonObject.put("list", list);
        return jsonObject;
    }

    /**
     * 生成h5地址
     *
     * @param myBean
     * @param params
     * @return
     * @throws Exception
     */

    public static String createScheduleEntryUrl(MyOutputBean myBean, Map<String, Object> params) throws Exception {
        String callBack = "";
        String baseUrl = MyConfig.getInstance().getUrl();
        FRLogger.getLogger().error("当前推送的信息：" + myBean.toString());
        FRLogger.getLogger().error("参数：" + params.toString());
        if (myBean.getLinkOpenType() == MessageUrlType.INNER.toInt()) {
            String taskName = CodeUtils.encodeURIComponent((String) params.get("taskName"));
            String savePath = URLEncoder.encode((String) params.get("saveDirectory"), "UTF-8");
            savePath = URLEncoder.encode(savePath, "UTF-8");
//            String savePath = (String) params.get("saveDirectory");
//            savePath = savePath.replaceAll("\\+", "%20");
            String showtype = (String) params.get("showType");
            int taskType = (Integer) params.get("taskType");
            String username = (String) params.get("username");
            callBack = baseUrl + "/schedule/result?taskName=" + taskName + "&username=" + username + "&path=" + savePath + "&showType=" + showtype + "&taskType=" + taskType + "&op=h5&terminal=H5";
//            callBack = CodeUtils.encodeURIComponent(callBack);
        } else {
//            callBack = CodeUtils.encodeURIComponent(myBean.getCustomizeLink());
            callBack = myBean.getCustomizeLink();
        }
        PushEntity entity = new PushEntity();
        entity.setOpenType(1);
        entity.setUrl(callBack);
        String id = UUID.randomUUID().toString();
        entity.setPushId(id);
        entity.setId(id);
        try {
            MyCoreDBAccess.getAccessor().runDMLAction(new DBAction<Object>() {
                @Override
                public Object run(DAOContext daoContext) throws Exception {
                    daoContext.getDAO(PushDao.class).add(entity);
                    return null;
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }

//        return baseUrl+"/plugin/public/com.fr.plugin.meixin.10/zch/pushBean?id="+id;
//        String url = baseUrl + "/plugin/public/com.fr.plugin.meixin.10/zch/login?callBack=" + callBack;
        return callBack;
//        return "https://www.baidu.com/s?ie=utf-8&f=3&rsv_bp=1&rsv_idx=1&tn=baidu&wd=%E6%88%90%E9%83%BD%E9%BA%BB%E8%BE%A3%E5%85%94%E5%A4%B4&rsv_pq=df6696af003bcde2&rsv_t=d4f5SCFCDvdN0%2BJOeALgYYCpRAeM3meoVqEvGtxz%2BWRE7s%2FDHKJ0XcbjNt8&rqlang=cn&rsv_dl=ts_0&rsv_enter=1&rsv_sug3=18&rsv_sug1=14&rsv_sug7=100&rsv_sug2=0&prefixsug=%25E6%2588%2590%25E9%2583%25BD%25E9%25BA%25BB%25E8%25BE%25A3&rsp=0&inputT=10785&rsv_sug4=10785";
    }

//    public static String toBase64(InputStream paramInputStream) {
//        byte[] arrayOfByte = IOUtils.inputStream2Bytes(paramInputStream);
//        return Base64.encode(arrayOfByte);
//    }

    private String getResultUrl(Map<String, Object> params) {
        ServerConfig.getInstance().getReportServletMapping();
        String url = MyConfig.getInstance().getUrl() + "/schedule/result?taskName=%s&username=%s&path=%s&showType=%s&taskType=%s";
        String taskName = CodeUtils.encodeURIComponent((String) params.get("taskName"));
        String saveDirectory = CodeUtils.encodeURIComponent((String) params.get("saveDirectory"));
        saveDirectory = saveDirectory.replaceAll("\\+", "%20");
        String showType = (String) params.get("showType");
        int taskType = (Integer) params.get("taskType");
        String username = (String) params.get("username");
        String callBack = String.format(url, taskName, username, saveDirectory, showType, taskType);
        PushEntity entity = new PushEntity();
        entity.setOpenType(1);
        entity.setUrl(callBack);
        String id = UUID.randomUUID().toString();
        entity.setPushId(id);
        entity.setId(id);
        String baseUrl = MyConfig.getInstance().getUrl();
        try {
            MyCoreDBAccess.getAccessor().runDMLAction(new DBAction<Object>() {
                @Override
                public Object run(DAOContext daoContext) throws Exception {
                    daoContext.getDAO(PushDao.class).add(entity);
                    return null;
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
//        return baseUrl+"/plugin/public/com.fr.plugin.meixin.10/zch/pushBean?id="+id;
        return callBack;
    }
}
