package com.fr.plugin.utils;

import com.fr.base.FRContext;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.config.MyConfig;
import com.fr.third.fasterxml.jackson.databind.JsonNode;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

public class PushUtils {
    private static final int NEWS_MESSAGE = 2;
    private static final int TERMINATE_TYPE = 107;
    private static final int LINKTYPE = 0;

//    public static void sendPush(JSONObject params) {
//        String url = "https://www.yunzhijia.com/pubacc/pubsend";
//        String s = WebUtils.sendJsonPost(url, params.toString());
//        JSONObject resp = new JSONObject(s);
//        String pubId = resp.getString("pubId");
//        FineLoggerFactory.getLogger().info("华发 推送成功:{}", pubId);
//    }
//
//    public static JSONObject genernateFrom(String no, String pubId, String pubsercet, String nonce, String time) {
//        JSONObject jsonFrom = new JSONObject();
//        jsonFrom.put("no", no);
//        jsonFrom.put("pub", pubId);
//        jsonFrom.put("nonce", nonce);
//        jsonFrom.put("time", time);
//        String pubtoken = sha(no, pubId, pubsercet, nonce, time);
//        jsonFrom.put("pubtoken", pubtoken);
//        return jsonFrom;
//    }

//    private static String sha(String... data) {
//        Arrays.sort(data);
//        String join = StringUtils.join(data);
//        String pubtoken = DigestUtils.sha1Hex(join);
//        return pubtoken;
//    }


    /**
     * {
     * "code": 0,
     * "msg": "成功",
     * "data": {
     * "errcode": 1,
     * "pushId": "",
     * "errmsg": "推送成功，推送回执：,推送到以下人员:,推送到以下部门:"
     * }
     *
     * @param pushObject
     * @param users
     */
    public static boolean push(String url, String pushSecret, String pushSysKey, String accessToken, PushObject pushObject, String users) {
        String body = JsonUtils.toJsonString(pushObject);
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("appKey", MyConfig.getInstance().getAppKey());
        params.put("sourceType", "todo");
        params.put("touser", users);
        params.put("timestamp", System.currentTimeMillis());
        params.put("accessToken", accessToken);
        String sysKey = pushSysKey;
        params.put("sysKey", sysKey);

        String sign = SignUtils.getSign(params, pushSecret);
        params.put("sign", sign);
        String param = SignUtils.pinParam(params);
        url = url + "?" + param;
        FRContext.getLogger().info("请求url:" + url);
        FRContext.getLogger().info("请求body:" + body);
        String s = sendPost(url, body);
        FRContext.getLogger().info("响应内容：" + s);

        if (com.fr.stable.StringUtils.isNotBlank(s)) {
            try {
                JsonNode jsonNode = JsonUtils.parseToJsonNode(s);
                String code = jsonNode.get("code").asText();
                if ("0".equalsIgnoreCase(code)) {
                    return true;
                }
            } catch (IOException e) {
            }
        }
        return false;
    }


    /**
     * 向指定 URL 发送POST方法的请求
     *
     * @param url   发送请求的 URL
     * @param param 请求参数，请求参数应该是 name1=value1&name2=value2 的形式。
     * @return 所代表远程资源的响应结果
     */
    public static String sendPost(String url, String param) {
        OutputStreamWriter out = null;
        BufferedReader in = null;
        StringBuilder result = new StringBuilder();
        try {
            URL realUrl = new URL(url);
            HttpURLConnection conn = null;
            conn = (HttpURLConnection) realUrl.openConnection();
            // 打开和URL之间的连接
            // 发送POST请求必须设置如下两行
            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setRequestMethod("POST");    // POST方法
            // 设置通用的请求属性
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("user-agent",
                    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            conn.setRequestProperty("Content-Type", "application/json");
            conn.connect();
            // 获取URLConnection对象对应的输出流
            out = new OutputStreamWriter(conn.getOutputStream(), StandardCharsets.UTF_8);
            // 发送请求参数
            out.write(param);
            // flush输出流的缓冲
            out.flush();
            // 定义BufferedReader输入流来读取URL的响应
            in = new BufferedReader(
                    new InputStreamReader(conn.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                result.append(line);
            }
        } catch (Exception e) {
        }
        //使用finally块来关闭输出流、输入流
        finally {
            try {
                if (out != null) {
                    out.close();
                }
                if (in != null) {
                    in.close();
                }
            } catch (IOException ex) {
            }
        }
        return result.toString();
    }

    public static String pushTextOrImage(String url, Object pushImageBean, String sysKey, String secrct) {
        String body = JsonUtils.toJsonString(pushImageBean);
        Map<String, Object> params = new HashMap<String, Object>();
        String accessToken = MyConfig.getInstance().getAccessToken();
        params.put("sysKey", sysKey);
        params.put("timestamp", System.currentTimeMillis());
        params.put("accessToken", accessToken);
        String sign = SignUtils.getSign(params, secrct);
        params.put("sign", sign);
        String param = SignUtils.pinParam(params);
        url = url + "?" + param;
        String s = sendPost(url, body);
        FineLoggerFactory.getLogger().error("推送图文响应:{}", s);
        FineLoggerFactory.getLogger().info("请求url:" + url);
        FineLoggerFactory.getLogger().info("请求body:" + body);
        if (com.fr.stable.StringUtils.isNotBlank(s)) {
            try {
                JsonNode jsonNode = JsonUtils.parseToJsonNode(s);
                String code = jsonNode.get("code").asText();
                if ("0".equalsIgnoreCase(code)) {
                    return jsonNode.get("data").get("msg_data_id").asText();
                }
            } catch (IOException e) {
            }
        }
        return null;
    }

    /**
     * 创建单图推送，
     * 流程1先创建image 2推送
     *
     * @param url
     * @param pushImageBean
     * @param sysKey
     * @param secrct
     * @return
     */
    public static String pushImage(String baseurl, String userId, String title, String sid, String imageUrl, String jumpUrl, int openType, String sysKey, String secrct) {
        Map<String, Object> params = new HashMap<String, Object>();
        String accessToken = MyConfig.getInstance().getAccessToken();
        params.put("sysKey", sysKey);
        params.put("timestamp", System.currentTimeMillis());
        params.put("accessToken", accessToken);
        String sign = SignUtils.getSign(params, secrct);
        params.put("sign", sign);
        String param = SignUtils.pinParam(params);
        String url=baseurl+"/mpm/v5/open/imgtext/upload";
        url = url + "?" + param;
        //创建图片
        JSONObject createImage = new JSONObject();
        createImage.put("sid", sid);
        createImage.put("title", title);
        createImage.put("jump_url", jumpUrl);
        createImage.put("cover_thumb_s", imageUrl);
        createImage.put("openType", openType);
        String createBody = createImage.toString();
        String s = sendPost(url, createBody);
        FineLoggerFactory.getLogger().error("创建图片响应:{}", s);
        FineLoggerFactory.getLogger().info("请求url:" + url);
        FineLoggerFactory.getLogger().info("请求body:" + createBody);
        String imageId = "";
        if (com.fr.stable.StringUtils.isNotBlank(s)) {
            try {
                JsonNode jsonNode = JsonUtils.parseToJsonNode(s);
                String code = jsonNode.get("code").asText();
                if ("0".equalsIgnoreCase(code)) {
                    imageId = jsonNode.get("msg_data_id").asText();
                }
            } catch (IOException e) {
            }
        }
        /**
         * 推送图片
         */
        url=baseurl+"/mpm/v5/open/imgtext/pushMsg";
        JSONObject pushImageBean = new JSONObject();
        pushImageBean.put("appKey", MyConfig.getInstance().getAppKey());
        JSONObject filter = new JSONObject();
        filter.put("user_id", userId);
        pushImageBean.put("filter", filter);
//        pushImageBean.put("sid", )
        JSONObject news = new JSONObject();
        news.put("media_id", imageId);
        pushImageBean.put("news", pushImageBean);
        params = new HashMap<String, Object>();
        params.put("sysKey", sysKey);
        params.put("timestamp", System.currentTimeMillis());
        params.put("accessToken", accessToken);
        sign = SignUtils.getSign(params, secrct);
        params.put("sign", sign);
        param = SignUtils.pinParam(params);
        url = url + "?" + param;
        String json = sendPost(url, pushImageBean.toString());
        FineLoggerFactory.getLogger().error("创建图片响应:{}", json);
        FineLoggerFactory.getLogger().info("请求url:" + url);
        FineLoggerFactory.getLogger().info("请求body:" + pushImageBean);
        if (com.fr.stable.StringUtils.isNotBlank(json)) {
            try {
                JsonNode jsonNode = JsonUtils.parseToJsonNode(json);
                String code = jsonNode.get("code").asText();
                if ("0".equalsIgnoreCase(code)) {
                    jsonNode.get("msg_id").asText();
                }
            } catch (IOException e) {
            }
        }
        return null;
    }


}
