package com.fr.plugin.web.hander.member;

import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.utils.SyncThread;
import com.fr.plugin.utils.WebHookUtil;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;

/**
 * 接受云之家回调接口
 */
public class SycHookHander extends BaseHttpHandler {

    @Override
    public RequestMethod getMethod() {
        return RequestMethod.GET;
    }

    @Override
    public String getPath() {
        return "/syn_hook";
    }

    @Override
    public boolean isPublic() {
        return true;
    }

    @Override
    public void handle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        String eid = httpServletRequest.getParameter("eid");
        String eventId = httpServletRequest.getParameter("eventId");
        String createTime = httpServletRequest.getParameter("createTime");
        String eventType = httpServletRequest.getParameter("eventType");
        Enumeration<String> names = httpServletRequest.getHeaderNames();
        Map<String ,String > header=new HashMap<>();
        while (names.hasMoreElements()) {
            String key = names.nextElement();
            String v = httpServletRequest.getHeader(key);
            header.put(key,v);
        }
        String contentBody = "eid=" + eid+ "&eventType=" + eventType + "&eventId=" + eventId+"&createTime="+createTime;
        FineLoggerFactory.getLogger().info(contentBody);
        Map<String,String> paramsMap = new TreeMap<String,String>();
        paramsMap.put("eid", eid);
        paramsMap.put("eventType", eventType);
        paramsMap.put("eventId", eventId);
        paramsMap.put("createTime", createTime);
        contentBody = mapToString(paramsMap);
        if(WebHookUtil.checkAuth("123", contentBody, header)){
            FineLoggerFactory.getLogger().info("接收到一个合法推送，内容为： "+contentBody);
            switch (eventType){
                case "user_enter":
                case "person_update":
                case "user_leave":
                case "user_phone_update":
                    new Thread(new SyncThread("user")).start();
                    break;
                case "org_add":
                case "org_update":
                case "org_admin":
                case "org_delete":
                case "person_role":
                    new Thread(new SyncThread("dep")).start();
                    break;
            }
            WebUtils.printAsString(httpServletResponse,"ok");
        }else{
            FineLoggerFactory.getLogger().info("接收到一个非法推送");
            WebUtils.printAsString(httpServletResponse,"ok");
        }
    }
    // 按key字段顺序排序，组装k1=v1&k2=v2形式
    private String mapToString(Map<String, String> map) {
        StringBuilder sb = new StringBuilder();
        Set<String> keys = map.keySet();
        for (String key : keys) {
            sb.append(key).append("=").append(map.get(key)).append("&");
        }
        if (sb.length() > 0) {
            return sb.substring(0, sb.length() - 1);
        } else {
            return sb.toString();
        }
    }
}
